//+------------------------------------------------------------------+
//|                                                 EnvelopesMA5.mq5 |
//|                     Copyright 2011, Computational Research, LLC. |
//|                                        http://www.firestrand.com |
//+------------------------------------------------------------------+
#property copyright "Copyright 2011, Computational Research, LLC."
#property link      "http://www.firestrand.com"
#property version   "1.00"
//+------------------------------------------------------------------+
//| Include                                                          |
//+------------------------------------------------------------------+
#include <Expert\Expert.mqh>
//--- available signals
#include <Expert\Signal\SignalTEMA.mqh>
#include <Expert\Signal\SignalEnvelopes.mqh>
//--- available trailing
#include <Expert\Trailing\TrailingParabolicSAR.mqh>
//--- available money management
#include <Expert\Money\MoneyFixedRisk.mqh>
//+------------------------------------------------------------------+
//| Inputs                                                           |
//+------------------------------------------------------------------+
//--- inputs for expert
input string             Expert_Title                 ="EnvelopesMA5"; // Document name
ulong                    Expert_MagicNumber           =18553;          // 
bool                     Expert_EveryTick             =false;          // 
//--- inputs for main signal
input int                Signal_ThresholdOpen         =9;             // Signal threshold value to open [0...100]
input int                Signal_ThresholdClose        =40;             // Signal threshold value to close [0...100]
input double             Signal_PriceLevel            =0.0;            // Price level to execute a deal
input double             Signal_StopLevel             =125.0;           // Stop Loss level (in points)
input double             Signal_TakeLevel             =377.0;           // Take Profit level (in points)
input int                Signal_Expiration            =4;              // Expiration of pending orders (in bars)
input int                Signal_TEMA_PeriodMA         =257;             // Triple Exponential Moving Average Period of averaging
input int                Signal_TEMA_Shift            =6;              // Triple Exponential Moving Average Time shift
input ENUM_APPLIED_PRICE Signal_TEMA_Applied          =PRICE_TYPICAL;  // Triple Exponential Moving Average Prices series
input double             Signal_TEMA_Weight           =0.614;            // Triple Exponential Moving Average Weight [0...1.0]
input int                Signal_Envelopes_PeriodMA    =3802;             // Envelopes(45,0,MODE_EMA,...) Period of averaging
input int                Signal_Envelopes_Shift       =10;              // Envelopes(45,0,MODE_EMA,...) Time shift
input ENUM_MA_METHOD     Signal_Envelopes_Method      =MODE_EMA;       // Envelopes(45,0,MODE_EMA,...) Method of averaging
input ENUM_APPLIED_PRICE Signal_Envelopes_Applied     =PRICE_TYPICAL;  // Envelopes(45,0,MODE_EMA,...) Prices series
input double             Signal_Envelopes_Deviation   =0.619;           // Envelopes(45,0,MODE_EMA,...) Deviation
input double             Signal_Envelopes_Weight      =0.586;            // Envelopes(45,0,MODE_EMA,...) Weight [0...1.0]
//--- inputs for trailing
input double             Trailing_ParabolicSAR_Step   =0.0064;           // Speed increment
input double             Trailing_ParabolicSAR_Maximum=0.242;            // Maximum rate
//--- inputs for money
input double             Money_FixRisk_Percent=20.9;           // Risk percentage
//+------------------------------------------------------------------+
//| Global expert object                                             |
//+------------------------------------------------------------------+
CExpert ExtExpert;
//+------------------------------------------------------------------+
//| Initialization function of the expert                            |
//+------------------------------------------------------------------+
int OnInit()
  {
//--- Initializing expert
   if(!ExtExpert.Init(Symbol(),Period(),Expert_EveryTick,Expert_MagicNumber))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing expert");
      ExtExpert.Deinit();
      return(-1);
     }
//--- Creating signal
   CExpertSignal *signal=new CExpertSignal;
   if(signal==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating signal");
      ExtExpert.Deinit();
      return(-2);
     }
//---
   ExtExpert.InitSignal(signal);
   signal.ThresholdOpen(Signal_ThresholdOpen);
   signal.ThresholdClose(Signal_ThresholdClose);
   signal.PriceLevel(Signal_PriceLevel);
   signal.StopLevel(Signal_StopLevel);
   signal.TakeLevel(Signal_TakeLevel);
   signal.Expiration(Signal_Expiration);
//--- Creating filter CSignalTEMA
   CSignalTEMA *filter0=new CSignalTEMA;
   if(filter0==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter0");
      ExtExpert.Deinit();
      return(-3);
     }
   signal.AddFilter(filter0);
//--- Set filter parameters
   filter0.PeriodMA(Signal_TEMA_PeriodMA);
   filter0.Shift(Signal_TEMA_Shift);
   filter0.Applied(Signal_TEMA_Applied);
   filter0.Weight(Signal_TEMA_Weight);
//--- Creating filter CSignalEnvelopes
   CSignalEnvelopes *filter1=new CSignalEnvelopes;
   if(filter1==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating filter1");
      ExtExpert.Deinit();
      return(-4);
     }
   signal.AddFilter(filter1);
//--- Set filter parameters
   filter1.PeriodMA(Signal_Envelopes_PeriodMA);
   filter1.Shift(Signal_Envelopes_Shift);
   filter1.Method(Signal_Envelopes_Method);
   filter1.Applied(Signal_Envelopes_Applied);
   filter1.Deviation(Signal_Envelopes_Deviation);
   filter1.Weight(Signal_Envelopes_Weight);
//--- Creation of trailing object
   CTrailingPSAR *trailing=new CTrailingPSAR;
   if(trailing==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating trailing");
      ExtExpert.Deinit();
      return(-5);
     }
//--- Add trailing to expert (will be deleted automatically))
   if(!ExtExpert.InitTrailing(trailing))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing trailing");
      ExtExpert.Deinit();
      return(-6);
     }
//--- Set trailing parameters
   trailing.Step(Trailing_ParabolicSAR_Step);
   trailing.Maximum(Trailing_ParabolicSAR_Maximum);
//--- Creation of money object
   CMoneyFixedRisk *money=new CMoneyFixedRisk;
   if(money==NULL)
     {
      //--- failed
      printf(__FUNCTION__+": error creating money");
      ExtExpert.Deinit();
      return(-7);
     }
//--- Add money to expert (will be deleted automatically))
   if(!ExtExpert.InitMoney(money))
     {
      //--- failed
      printf(__FUNCTION__+": error initializing money");
      ExtExpert.Deinit();
      return(-8);
     }
//--- Set money parameters
   money.Percent(Money_FixRisk_Percent);
//--- Check all trading objects parameters
   if(!ExtExpert.ValidationSettings())
     {
      //--- failed
      ExtExpert.Deinit();
      return(-9);
     }
//--- Tuning of all necessary indicators
   if(!ExtExpert.InitIndicators())
     {
      //--- failed
      printf(__FUNCTION__+": error initializing indicators");
      ExtExpert.Deinit();
      return(-10);
     }
//--- ok
   return(0);
  }
//+------------------------------------------------------------------+
//| Deinitialization function of the expert                          |
//+------------------------------------------------------------------+
void OnDeinit(const int reason)
  {
   ExtExpert.Deinit();
  }
//+------------------------------------------------------------------+
//| "Tick" event handler function                                    |
//+------------------------------------------------------------------+
void OnTick()
  {
   ExtExpert.OnTick();
  }
//+------------------------------------------------------------------+
//| "Trade" event handler function                                   |
//+------------------------------------------------------------------+
void OnTrade()
  {
   ExtExpert.OnTrade();
  }
//+------------------------------------------------------------------+
//| "Timer" event handler function                                   |
//+------------------------------------------------------------------+
void OnTimer()
  {
   ExtExpert.OnTimer();
  }
//+------------------------------------------------------------------+
double OnTester()
{
   double result = 0.0;
   double profitFactor = TesterStatistics(STAT_PROFIT_FACTOR);
   //Throw out a DBL_MAX profit factor and 0 trades
   if(profitFactor == DBL_MAX || TesterStatistics(STAT_TRADES) == 0)
      return -1.0e3;
   
   result += Scale(profitFactor,0,10);
   result += ApproxTanh(TesterStatistics(STAT_SHARPE_RATIO));
   //Weigh the Profit more
   result += 3.0 * Scale(TesterStatistics(STAT_PROFIT)/TesterStatistics(STAT_INITIAL_DEPOSIT),-5,5,-1,1);
   
   //Minimize 
   //TesterStatistics(STAT_EQUITY_DD_RELATIVE);
   
   return result * 100.0;
}
double ApproxTanh(double x)
{
   //This is based on Lambert continued fraction taken to 7 divisions and optimized
   double x2 = x*x;
   double ax = (((x2+378.0)*x2+17325.0)*x2+135135.0)*x;
   double bx = ((28.0*x2+3150.0)*x2+62370.0)*x2+135135.0;
   return ax/bx;
}
//- Will scale x from 0-1 using passed min and max
double Scale(double x, double min, double max)
{
   return (x - min) / (max - min);
}
double Scale(double x, double min, double max, double newMin, double newMax)
{
   return ((x - min)/(max - min))*(newMax - newMin) + newMin;
}